function NODES = OMNICONT_compress_HSS_dsym_brute(C,nbox_max,acc,flag_pot,params)

% Compute the tree strcture.
ntot   = size(C,2);
NODES  = LOCAL_get_tree(C,nbox_max);
nboxes = size(NODES,2);

% Compress all nodes, going from smaller to larger.
for ibox = nboxes:(-1):2
  ind_offd = [1:(NODES{6,ibox}-1),(NODES{6,ibox}+NODES{7,ibox}):ntot];
  if (NODES{5,ibox}==0) % ibox has no sons.
    indskel = NODES{6,ibox} - 1 + (1:NODES{7,ibox});
  elseif (NODES{5,ibox}==1) % ibox has precisely one son
    ison1   = NODES{4,ibox}(1);
    indskel = NODES{20,ison1};
  elseif (NODES{5,ibox}==2) % ibox has precisely two sons
    ison1   = NODES{4,ibox}(1);
    ison2   = NODES{4,ibox}(2);
    indskel = [NODES{20,ison1},NODES{20,ison2}];
  end
  A21 = OMNICONT_construct_A_offd(C,ind_offd,indskel,flag_pot,params);
  A12 = OMNICONT_construct_A_offd(C,indskel,ind_offd,flag_pot,params);
  % Compute the skeletons.
  [T,J] = id_decomp([A21;A12'],acc);
  k     = size(T,1);
  % Record the skeletons:
  NODES{20,ibox} = indskel(J(1:k));
  NODES{30,ibox} = T;
  NODES{31,ibox} = J;
  % Record nskel and kskel
  NODES{ 8,ibox} = length(indskel);
  NODES{ 9,ibox} = k;
end

% Construct the matrices representing self-interactions on the leaves.
for ibox = nboxes:(-1):2
  if (NODES{5,ibox}==0) % ibox is a leaf
    ind = NODES{6,ibox} - 1 + (1:NODES{7,ibox});
    NODES{40,ibox} = OMNICONT_construct_A_diag(C,ind,flag_pot,params);
  end
end

% Construct the matrices for sibling interactions.
for ibox = nboxes:(-1):1
  if (NODES{5,ibox}==2) % ibox has precisely two sons
    ison1 = NODES{4,ibox}(1);
    ison2 = NODES{4,ibox}(2);
    NODES{46,ison1} = OMNICONT_construct_A_offd(C,NODES{20,ison1},NODES{20,ison2},flag_pot,params);
    NODES{46,ison2} = OMNICONT_construct_A_offd(C,NODES{20,ison2},NODES{20,ison1},flag_pot,params);
  end
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function NODES = LOCAL_get_tree(C,nbox_max)

ntot = size(C,2);

NODES_0 = cell(10,ntot);

% Construct the top node.
NODES_0{1,1} = [1,ntot];
NODES_0{2,1} = 0;
NODES_0{3,1} = -1;
NODES_0{4,1} = [];
NODES_0{5,1} = 0;
NODES_0{6,1} = 1;
NODES_0{7,1} = ntot;
NODES_0{8,1} = -1; 
NODES_0{9,1} = -1;

% Create smaller nodes via hierarchical subdivision.
% We sweep one level at a time.
ibox_last =  0;
ncreated  =  1;
while (ncreated > 0)
  ibox_first = ibox_last + 1;
  ibox_last  = ibox_last + ncreated;
  ncreated   = 0;
  for ibox = ibox_first:ibox_last
    nbox = NODES_0{7,ibox};
    if (nbox > nbox_max)
      nhalf              = floor(NODES_0{7,ibox}/2);
      if (nhalf > 0)
        ison1            = ibox_last + ncreated + 1;
        NODES_0{4,ibox}  = [NODES_0{4,ibox},ison1];   % Add a child to ibox.
        NODES_0{5,ibox}  = NODES_0{5,ibox}+1;         % Add a child to ibox.
        NODES_0{2,ison1} = NODES_0{2,ibox}+1;         % Set the level           of the child.
        NODES_0{3,ison1} = ibox;                      % Set the parent          of the child.
        NODES_0{4,ison1} = [];                        % Set the child list      of the child.
        NODES_0{5,ison1} = 0;                         % Set the child list      of the child.
        NODES_0{6,ison1} = NODES_0{6,ibox};           % Set the left-most index of the child. 
        NODES_0{7,ison1} = nhalf;                     % Set the number of nodes of the child.
        ncreated         = ncreated + 1;
      end
      if (nhalf < NODES_0{7,ibox})
        ison2            = ibox_last + ncreated + 1;  
        NODES_0{4,ibox}  = [NODES_0{4,ibox},ison2];   % Add a child to ibox.
        NODES_0{5,ibox}  = NODES_0{5,ibox}+1;         % Add a child to ibox. 
        NODES_0{2,ison2} = NODES_0{2,ibox}+1;         % Set the level           of the child.   
        NODES_0{3,ison2} = ibox;                      % Set the parent          of the child.
        NODES_0{4,ison2} = [];                        % Set the child list      of the child. 
        NODES_0{5,ison2} = 0;                         % Set the child list      of the child.   
        NODES_0{6,ison2} = NODES_0{6,ibox} + nhalf;   % Set the left-most index of the child.
        NODES_0{7,ison2} = NODES_0{7,ibox} - nhalf;   % Set the number of nodes of the child. 
        ncreated = ncreated + 1;
      end
    end
  end
end

% This whole section is devoted to the simple task of
% decreasing the size of NODES_0.
% The operation NODES_0 = NODES_0{:,1:nboxes} apparently
% does not work for "cell" structures.
nboxes = ibox_last;
NODES  = cell(50,nboxes);
for ibox = 1:nboxes
  for j = 1:size(NODES_0,1)
    NODES{j,ibox} = NODES_0{j,ibox};
  end
end

return
